var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;
exports.getProcessScriptMethAst = getProcessScriptMethAst;
exports.getGetLocationMethAst = getGetLocationMethAst;
exports.getSetLocationMethAst = getSetLocationMethAst;
exports.getSetMethAst = getSetMethAst;
exports.getCallMethodMthAst = getCallMethodMthAst;
exports.getGetMethAst = getGetMethAst;
exports.getGetComputedMethAst = getGetComputedMethAst;
exports.getSetComputedMethAst = getSetComputedMethAst;
exports.getConcatOperatorAst = getConcatOperatorAst;
exports.getDocumentWriteArgAst = getDocumentWriteArgAst;
exports.getDocumentWriteStatementIndices = getDocumentWriteStatementIndices;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _instructions = require('./instructions');

var instructs = _interopRequireWildcard(_instructions);

var _parsingTools = require('./parsing-tools');

function getProcessScriptMethAst(args) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.PROCESS_SCRIPT_METH_NAME
        },

        arguments: [args[0]]
    };
}

function getGetLocationMethAst() {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.GET_LOCATION_METH_NAME
        },

        arguments: [{
            type: _parsingTools.Syntax.Identifier,
            name: 'location'
        }]
    };
}

function getSetLocationMethAst(value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.MemberExpression,
            computed: false,

            object: {
                type: _parsingTools.Syntax.FunctionExpression,
                id: null,
                params: [],
                defaults: [],

                body: {
                    type: _parsingTools.Syntax.BlockStatement,
                    body: [{
                        type: _parsingTools.Syntax.ReturnStatement,

                        argument: {
                            type: _parsingTools.Syntax.LogicalExpression,
                            operator: '||',

                            left: {
                                type: _parsingTools.Syntax.CallExpression,

                                callee: {
                                    type: _parsingTools.Syntax.Identifier,
                                    name: instructs.SET_LOCATION_METH_NAME
                                },

                                arguments: [{
                                    type: _parsingTools.Syntax.Identifier,
                                    name: 'location'
                                }, value]
                            },

                            right: {
                                type: _parsingTools.Syntax.AssignmentExpression,
                                operator: '=',

                                left: {
                                    type: _parsingTools.Syntax.Identifier,
                                    name: 'location'
                                },

                                right: value
                            }
                        }
                    }]
                },

                rest: null,
                generator: false,
                expression: false
            },

            property: {
                type: _parsingTools.Syntax.Identifier,
                name: 'apply'
            }
        },

        arguments: [{
            type: _parsingTools.Syntax.ThisExpression
        }]
    };
}

function getSetMethAst(propertyName, obj, value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.SET_PROPERTY_METH_NAME
        },

        arguments: [obj, {
            type: _parsingTools.Syntax.Literal,
            value: propertyName,
            raw: '"' + propertyName + '"'
        }, value]
    };
}

function getCallMethodMthAst(owner, meth, args) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.CALL_METHOD_METH_NAME
        },

        arguments: [owner, meth, {
            type: _parsingTools.Syntax.ArrayExpression,
            elements: args
        }]
    };
}

function getGetMethAst(propertyName, owner) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.GET_PROPERTY_METH_NAME
        },

        arguments: [owner, {
            type: _parsingTools.Syntax.Literal,
            value: propertyName,
            raw: '"' + propertyName + '"'
        }]
    };
}

function getGetComputedMethAst(property, owner) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.GET_PROPERTY_METH_NAME
        },

        arguments: [owner, property]
    };
}

function getSetComputedMethAst(property, owner, value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: instructs.SET_PROPERTY_METH_NAME
        },

        arguments: [owner, property, value]
    };
}

function getConcatOperatorAst(left, right) {
    return {
        type: _parsingTools.Syntax.AssignmentExpression,
        operator: '=',
        left: left,

        right: {
            type: _parsingTools.Syntax.BinaryExpression,
            operator: '+',
            left: left,
            right: right
        }
    };
}

function getDocumentWriteArgAst(arg) {
    return {
        type: _parsingTools.Syntax.Literal,
        value: arg,
        raw: '\'' + arg + '\''
    };
}

function getDocumentWriteStatementIndices(statements) {
    var indices = [];

    var isExpressionStatement = function (statement) {
        return statement.type === _parsingTools.Syntax.ExpressionStatement;
    };
    var isCallStatement = function (statement) {
        return statement.expression.type === _parsingTools.Syntax.CallExpression;
    };
    var isMember = function (statement) {
        return statement.expression.callee.type === _parsingTools.Syntax.MemberExpression;
    };
    var isDocumentWrite = function (statement) {
        return statement.expression.callee.property.name === 'write' || statement.expression.callee.property.name === 'writeln';
    };

    for (var i = 0; i < statements.length; i++) {
        var statement = statements[i];

        if (isExpressionStatement(statement) && isCallStatement(statement) && isMember(statement) && isDocumentWrite(statement)) indices.push(i);
    }

    return indices;
}
//# sourceMappingURL=templates.js.map
