var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

exports.__esModule = true;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _const = require('../const');

var IS_STYLESHEET_PROCESSED_REG_EX = new RegExp('^\\s*' + _const.IS_STYLESHEET_PROCESSED_COMMENT.replace(/\/|\*/g, '\\$&'));

var SOURCE_MAP_REG_EX = /#\s*sourceMappingURL\s*=\s*[^\s]+(\s|\*\/)/i;
var CSS_URL_PROPERTY_VALUE_PATTERN = /(url\s*\(\s*)(?:(')([^\s']*)(')|(")([^\s"]*)(")|([^\s\)]*))(\s*\))|(@import\s+)(?:(')([^\s']*)(')|(")([^\s"]*)("))/g;

var StyleProcessor = (function () {
    function StyleProcessor() {
        _classCallCheck(this, StyleProcessor);
    }

    StyleProcessor.prototype.process = function process(css, urlReplacer, isStylesheetTable) {
        var isStylesheetProcessed = IS_STYLESHEET_PROCESSED_REG_EX.test(css);

        if (typeof css === 'string' && !isStylesheetProcessed) {
            var prefix = isStylesheetTable ? _const.IS_STYLESHEET_PROCESSED_COMMENT + '\n' : '';

            // Replace :hover pseudo class
            css = css.replace(/\s*:\s*hover(\W)/gi, '[' + _const.HOVER_PSEUDO_CLASS_ATTR + ']$1');

            // Remove source map directive
            css = css.replace(SOURCE_MAP_REG_EX, '$1');

            // NOTE: replace URLs in css rules with the proxy URLs.
            return prefix + this._replaceStylsheetUrls(css, urlReplacer);
        }

        return css;
    };

    StyleProcessor.prototype.cleanUp = function cleanUp(css, parseProxyUrl, formatUrl) {
        if (typeof css === 'string') {
            css = css.replace(new RegExp('\\[' + _const.HOVER_PSEUDO_CLASS_ATTR + '\\](\\W)', 'ig'), ':hover$1');

            return this._replaceStylsheetUrls(css, function (url) {
                var originUrlObj = parseProxyUrl(url);

                if (originUrlObj) return formatUrl(originUrlObj.originResourceInfo);

                return url;
            });
        }

        return css;
    };

    StyleProcessor.prototype._replaceStylsheetUrls = function _replaceStylsheetUrls(css, processor) {
        return css.replace(CSS_URL_PROPERTY_VALUE_PATTERN, function () {
            var prefix = arguments[1] || arguments[10];
            var openQuote = arguments[2] || arguments[5] || arguments[11] || arguments[14] || '';
            var url = arguments[3] || arguments[6] || arguments[8] || arguments[12] || arguments[15];
            var closeQuote = arguments[4] || arguments[7] || arguments[13] || arguments[16] || '';
            var postfix = arguments[9] || '';

            return url ? prefix + openQuote + processor(url) + closeQuote + postfix : arguments[0];
        });
    };

    return StyleProcessor;
})();

exports.default = new StyleProcessor();
module.exports = exports.default;
//# sourceMappingURL=style.js.map
