var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
exports.isPage = isPage;
exports.isCSSResource = isCSSResource;
exports.isScriptResource = isScriptResource;
exports.isManifest = isManifest;
exports.isJSON = isJSON;
exports.decodeContent = decodeContent;
exports.encodeContent = encodeContent;

var _iconvLite = require('iconv-lite');

var _iconvLite2 = _interopRequireDefault(_iconvLite);

var _promisify = require('./promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var gzip = _promisify2.default(_zlib2.default.gzip);
var deflate = _promisify2.default(_zlib2.default.deflate);
var gunzip = _promisify2.default(_zlib2.default.gunzip);
var inflate = _promisify2.default(_zlib2.default.inflate);
var inflateRaw = _promisify2.default(_zlib2.default.inflateRaw);

// Const
var JSON_MIME = 'application/json';
var MANIFEST_MIME = 'text/cache-manifest';
var CSS_MIME = 'text/css';

var PAGE_MIMES = ['text/html', 'application/xhtml+xml', 'application/xml', 'application/x-ms-application'];

var SCRIPT_MIMES = ['application/javascript', 'text/javascript', 'application/x-javascript'];

// Content type

function isPage(header) {
    header = header.toLowerCase();

    return PAGE_MIMES.some(function (mime) {
        return header.indexOf(mime) > -1;
    });
}

function isCSSResource(contentTypeHeader, acceptHeader) {
    return contentTypeHeader.toLowerCase().indexOf(CSS_MIME) > -1 || acceptHeader.toLowerCase() === CSS_MIME;
}

function isScriptResource(contentTypeHeader, acceptHeader) {
    contentTypeHeader = contentTypeHeader.toLowerCase();
    acceptHeader = acceptHeader.toLowerCase();

    return SCRIPT_MIMES.some(function (mime) {
        return contentTypeHeader.indexOf(mime) > -1;
    }) || SCRIPT_MIMES.indexOf(acceptHeader) > -1;
}

function isManifest(contentTypeHeader) {
    return contentTypeHeader.toLowerCase().indexOf(MANIFEST_MIME) > -1;
}

function isJSON(contentTypeHeader) {
    return contentTypeHeader.toLowerCase().indexOf(JSON_MIME) > -1;
}

// Encoding / decoding

// NOTE: IIS has a bug then it sends 'raw deflate' compressed
// data for 'Deflate' Accept-Encoding header.
// (see: http://zoompf.com/2012/02/lose-the-wait-http-compression)
function inflateWithFallback(data) {
    return _regeneratorRuntime.async(function inflateWithFallback$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.prev = 0;
                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(inflate(data));

            case 3:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 6:
                context$1$0.prev = 6;
                context$1$0.t0 = context$1$0['catch'](0);

                if (!(context$1$0.t0.code === 'Z_DATA_ERROR')) {
                    context$1$0.next = 12;
                    break;
                }

                context$1$0.next = 11;
                return _regeneratorRuntime.awrap(inflateRaw(data));

            case 11:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 12:
                throw context$1$0.t0;

            case 13:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[0, 6]]);
}

function decodeContent(content, encoding, charset) {
    return _regeneratorRuntime.async(function decodeContent$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                if (!(encoding === 'gzip')) {
                    context$1$0.next = 6;
                    break;
                }

                context$1$0.next = 3;
                return _regeneratorRuntime.awrap(gunzip(content));

            case 3:
                content = context$1$0.sent;
                context$1$0.next = 10;
                break;

            case 6:
                if (!(encoding === 'deflate')) {
                    context$1$0.next = 10;
                    break;
                }

                context$1$0.next = 9;
                return _regeneratorRuntime.awrap(inflateWithFallback(content));

            case 9:
                content = context$1$0.sent;

            case 10:

                charset.fromBOM(content);

                return context$1$0.abrupt('return', _iconvLite2.default.decode(content, charset.get()));

            case 12:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}

function encodeContent(content, encoding, charset) {
    return _regeneratorRuntime.async(function encodeContent$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                content = _iconvLite2.default.encode(content, charset.get(), { addBOM: charset.isFromBOM() });

                if (!(encoding === 'gzip')) {
                    context$1$0.next = 3;
                    break;
                }

                return context$1$0.abrupt('return', gzip(content));

            case 3:
                if (!(encoding === 'deflate')) {
                    context$1$0.next = 5;
                    break;
                }

                return context$1$0.abrupt('return', deflate(content));

            case 5:
                return context$1$0.abrupt('return', content);

            case 6:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
}
//# sourceMappingURL=content.js.map
